# -*- coding: utf-8 -*-
# Brunie Nicolas, War Of Blood

import pygame
from Common import load_png, Object
from Weapons import Rifle
from Blast import Movable
from Particle import ComplexParticleSystem

class CaisseBase:
  def __init__(self,(x,y,cx,cy),pe):
    self.pos = x,y
    self.rect.center = x,y
    self.globalPos = x + cx,y+cy
    self.direction = 0,0
    self.scale = 1.0
    self.gravity = 3
    self.structurePoint = 1000
    self.img = "caisse.png"
    self.image_origine,r = load_png("caisse.png")
    self.angle = 0

  def getDescription(self):
    """ pour le LevelDesigner """
    return "caisse" 

  def forcedMouvement(self,dx,dy):
    self.direction = dx,dy
    dx,dy = int(dx), int(dy)
    self.rect.move_ip(dx,dy)
    self.updateGlobalPosition(dx,dy)

  def updateGlobalPosition(self,dx,dy):
    gpx,gpy = self.globalPos
    self.globalPos = gpx + dx, gpy + dy

  def deplacement(self,x,y,cameraMove=None):
    self.rect.move_ip(int(x),int(y))

  def destruct(self):
    self.kill()  # on supprime de tous les groupes attachés



class Caisse(pygame.sprite.Sprite,Movable,CaisseBase):
  def __init__(self,obstacleGroup,persoGroup,(x,y,cx,cy),pe):
    pygame.sprite.Sprite.__init__(self)
    Movable.__init__(self,pe,40)
    if pe != None: print "PEid %d " % self.PEid
    self.image, self.rect = load_png("caisse.png")
    CaisseBase.__init__(self,(x,y,cx,cy),pe)
    self.obstacleGroup = obstacleGroup
    self.persoGroup = persoGroup

  def rebuild(self):
    self.solid = 1
    self.image,a = load_png("caisse.png")


  def sustentation(self):
    print "sustentation"
    self.gravity = 0

  def unSustentation(self):
    print "unsustentation"
    self.gravity = 3

  def looseLife2(self,n):
    print "caisse %d" % self.structurePoint
    self.structurePoint += n
    if self.structurePoint < 0:
      self.destruct()

  def destruct(self):
    self.image,a = load_png("caisse3.png")
    self.solid = 0

  def update(self,time):
    dx,dy = self.direction
    dx -= 0.1
    dy += self.gravity # chute par gravité
    if self.physicEngine.tryForcedMouvement(self,dx,dy,degat=100):
      ndx,ndy = dx,dy
    else:
      if dy > 22:
        self.destruct() # print "destruction de caisse"
      ndx,ndy = 0,0
    self.direction = ndx,ndy

class CaisseRessource(pygame.sprite.Sprite,Movable,CaisseBase):
  def __init__(self,obstacleGroup,persoGroup,(x,y,cx,cy),pe,type,img,pg):
    pygame.sprite.Sprite.__init__(self)
    Movable.__init__(self,pe,40)
    print "PEid %d " % self.PEid
    self.image, self.rect = load_png(img)
    CaisseBase.__init__(self,(x,y,cx,cy),pe)
    self.obstacleGroup = obstacleGroup
    self.persoGroup = persoGroup
    self.munition = 10
    self.type = type
    self.timeReapparition = 30000
    self.tmp = 30000
    self.state = "normal"
    self.particleGestionner = pg


  def update(self,time):
    if self.state != "normal":
      self.tmp -= time
      if self.tmp < 0:
        self.state = "normal"
        return 1
      else:
        return 0
    dx,dy = self.direction
    dx -= 0.1
    dy += self.gravity # chute par gravité
    if self.physicEngine.tryForcedMouvement(self,dx,dy,degat=100):
      ndx,ndy = dx,dy
    else:
      ndx,ndy = 0,0
    self.direction = ndx,ndy
    for perso in self.persoGroup:
      if perso.rect.colliderect(self.rect):
        perso.addAmunition(self.munition,self.type)
        x,y = self.rect.center
        y -= 100
        particle = ComplexParticleSystem(self.particleGestionner.screen,(x,y),1000,"+10.png", 1.0,500) 
        self.particleGestionner.attach(particle)
        self.state = "invisible"
        self.tmp = self.timeReapparition
        return 0
    return 1


class CaisseSoin(pygame.sprite.Sprite,Movable,CaisseBase):
  def __init__(self,obstacleGroup,persoGroup,(x,y,cx,cy),pe,pg):
    pygame.sprite.Sprite.__init__(self)
    Movable.__init__(self,pe,40)
    print "PEid %d " % self.PEid
    self.image, self.rect = load_png("caisse_soin.png")
    CaisseBase.__init__(self,(x,y,cx,cy),pe)
    self.obstacleGroup = obstacleGroup
    self.persoGroup = persoGroup
    self.soin = 400
    self.timeReapparition = 30000
    self.tmp = 30000
    self.state = "normal"
    self.particleGestionner = pg


  def update(self,time):
    if self.state != "normal":
      self.tmp -= time
      if self.tmp < 0:
        self.state = "normal"
        return 1
      else:
        return 0
    dx,dy = self.direction
    dx -= 0.1
    dy += self.gravity # chute par gravité
    if self.physicEngine.tryForcedMouvement(self,dx,dy,degat=100):
      ndx,ndy = dx,dy
    else:
      ndx,ndy = 0,0
    self.direction = ndx,ndy
    for perso in self.persoGroup:
      if perso.rect.colliderect(self.rect):
        perso.addLife(self.soin)
        x,y = self.rect.center
        y -= 100
        particle = ComplexParticleSystem(self.particleGestionner.screen,(x,y),1000,"+10%.png", 1.0,500) 
        self.particleGestionner.attach(particle)
        # self.destruct()
        self.state = "invisible"
        self.tmp = self.timeReapparition
        return 0
    return 1

class CaisseRifle(pygame.sprite.Sprite,Movable,CaisseBase):
  def __init__(self,obstacleGroup,persoGroup,(x,y,cx,cy),pe,pg,targetGroup):
    pygame.sprite.Sprite.__init__(self)
    Movable.__init__(self,pe,40)
    print "PEid %d " % self.PEid
    self.image, self.rect = load_png("caisse_rifle.png")
    CaisseBase.__init__(self,(x,y,cx,cy),pe)
    self.obstacleGroup = obstacleGroup
    self.persoGroup = persoGroup
    self.soin = 400
    self.timeReapparition = 30000
    self.tmp = 30000
    self.state = "normal"
    self.particleGestionner = pg
    self.targetGroup = targetGroup


  def update(self,time):
    if self.state != "normal":
      self.tmp -= time
      if self.tmp < 0:
        self.state = "normal"
        return 1
      else:
        return 0
    dx,dy = self.direction
    dx -= 0.1
    dy += self.gravity # chute par gravité
    if self.physicEngine.tryForcedMouvement(self,dx,dy,degat=100):
      ndx,ndy = dx,dy
    else:
      ndx,ndy = 0,0
    self.direction = ndx,ndy
    for perso in self.persoGroup:
      if perso.rect.colliderect(self.rect):
        rifle = Rifle(self.particleGestionner,self.targetGroup)
        perso.attach(rifle,(-16,-10),(16,-10),Object((0,0),"riflem16.png"))     
        print "attach rifle"
        # perso.addLife(self.soin)
        x,y = self.rect.center
        y -= 100
        particle = ComplexParticleSystem(self.particleGestionner.screen,(x,y),1000,"+10%.png", 1.0,500) 
        self.particleGestionner.attach(particle)
        # self.destruct()
        self.state = "invisible"
        self.tmp = self.timeReapparition
        return 0
    return 1

