#! /usr/bin/python
# -*- coding: utf-8 -*-
#
# moteur physique Blast (simplissime), Brunie nicolas
# version 0.2
#
# Utilisation dans War Of Blood , Brunie nicolas

from Common import vectdist2
from Angle import cangle, drawVector
import math

class Force:
  def __init__(self,name,dx,dy,pe,type=0):
    """ initialisation de la force
      - type = 0 : force basique
      - type = 1 : force qui annule les composantes parallèles à sa direction
      - type = 2 : force à usage unique, une fois prise en compte, cesse d'être utilisée """
    self.type = type
    self.direction = dx,dy
    self.physicEngine = pe
    self.name = name
    if pe != None: self.id = pe.getStrengthId()
    #print "force %d %d" % self.direction
    self.liste = []

  def addTo(self,l):
    l.append(self)
    self.liste.append(l)

  def get(self):
    pass

  def changeDirection(self,(dx,dy)):
    self.direction = dx,dy
  
  def projection(self,force2):
    """ on soustrait de force2 la composante tangente à self """
    fx,fy = force2
    f1x,f2y = self.direction
    norme = math.sqrt(f1x**2+f2y**2)
    vx,vy = f1x / norme , f2y / norme
    angle1 = cangle(fx,fy)
    angle2 = cangle(f1x,f2y)
    c = abs(math.cos((angle1 - angle2) / 360.0 * 2 * math.pi))
    tx,ty = fx * c, fy * c  # projection tangeante
    fx,fy = fx - tx, fy - ty
    return fx,fy


class Movable:
  def __init__(self,PE,w,type=0):
    """ initialisation de la classe des objets qui peuvent bouger """
    if PE != None: self.PEid = PE.getId()
    self.physicEngine = PE
    self.weight = w
    self.forces = [[Force("poids",0,w,PE)],[],[]] # type0 et type1 type2
    self.solid = 1
    self.type = type # 1 pour les persos

class UnMovable:
  def __init__(self,PE,w):
    self.PEid = PE.getId()
    self.physicEngine = PE
    self.forces = []
    self.weight = w

class BlastPhysicEngine:
  def __init__(self):
    self.obstacleGroup = []
    self.movableGroup = []
    self.gid = 0
    self.fid = 0 # id des forces

  def accelerationCalculus(self,objet,time):
    (forceList,forceListType1,forceType2) = objet.forces
    ax,ay = 0,0
    for force in forceType2:
      fx,fy = force.direction
      ax += fx
      ay += fy
    for force in forceList:
      fx,fy = force.direction
      ax += fx
      ay += fy
    for force in forceListType1:
      ax,ay = force.projection((ax,ay))
    ax *= time / 400.0
    ay *= time / 400.0
    objet.forces[2] = [] # on supprime les forces à usage unique
    return ax,ay

  def getStrengthId(self):
    id = self.fid
    self.fid += 1
    return id

  def addObstacleGroup(self,group):
    self.obstacleGroup.append(group)

  def addMovableGroup(self,group):
    self.movableGroup.append(group)

  def getId(self):
    self.gid += 1
    return self.gid

  def getGroundDistance(self,object,rect,dy=0,total=1):
    """ on calcule les distances par rapport au element immobile """
    # pr = perso.rect
    pb,pl,pr = rect.bottom,rect.left,rect.right
    max = 100000
    for group in self.obstacleGroup:
      for ob in group.sprites():
        obr = ob.rect
        if rect.colliderect(obr): return -1
        ot,ol,od = obr.top,obr.left,obr.right
        if ol <= pl <= od or ol <= pr <= od:
          if 0 <= abs(pb - ot) < max: max = abs(pb-ot)
    if total:
      for group in self.movableGroup:
        for ob in group.sprites():
          if object.type == 1 and ob.type == 1: continue
          obr = ob.rect
          if object.PEid == ob.PEid or not ob.solid: continue     # ATTENTION
          if dy:
            if rect.colliderect(obr): return -1
          ot,ol,od = obr.top,obr.left,obr.right
          if ol <= pl <= od or ol <= pr <= od or pl < ol < pr:
            if 0 <= abs(pb - ot) < max: max = abs(pb-ot)
    return max

  def lineOfSight(self,ob1,ob2):
    # equation de la droite a X + b Y + c = 0
    # 
    for ob in self.obstacleGroup:
      pass
      

  def mouvement(self,object,dx,dy,caisse=0,degat=0):
    """ object veut bouger dans la direction dx, dy 
    on vérifie qu'il le peut et on effectue le mouvement
      le cas échéant """
    id = object.PEid
    rx = object.rect.move(int(dx),0)
    ry = object.rect.move(0,int(dy))
    rect = object.rect.move(int(dx),int(dy))
    distx = self.getGroundDistance(object,rx,total=1)
    disty = self.getGroundDistance(object,ry,dy=caisse,total=1) # caisse si object n'est pas une caisse
    ndx,ndy = dx,dy
    if distx <= 3:
      ndx = 0
    if disty <= 3:
      ndy = 0
    type = object.type
    if type == 0:
      for group in self.movableGroup:
        for ob in group.sprites():
          obr = ob.rect
          if ob.PEid == id or not ob.solid: continue
          if rect.colliderect(obr):
            if degat: ob.looseLife2(-10000)
            if self.tryForcedMouvement(ob,int(ndx),int(ndy)) == 0:
              return 0,0
    elif type == 1:
      for group in self.movableGroup:
        for ob in group.sprites():
          obr = ob.rect
          if ob.PEid == id or not ob.solid: continue
          if rect.colliderect(obr) and ob.type != 1:
            if degat: ob.looseLife2(-10000)
            if self.tryForcedMouvement(ob,int(ndx),int(ndy)) == 0:
              return 0,0
    return ndx,ndy


  def tryForcedMouvement(self,object,dx,dy,level=0,degat=0):
    if level > 5: return 0
    """ si object bouge retourne 1 sinon 0 """
    id = object.PEid
    rx = object.rect.move(dx,0)
    ry = object.rect.move(0,dy)
    distx = self.getGroundDistance(object,rx,total=0)
    disty = self.getGroundDistance(object,ry,total=0)
    rect = object.rect.move(int(dx),int(dy))
    type = object.type
    if distx <= 3:
      return 0
    if disty <= 3:
      return 0
    for group in self.movableGroup:
      for ob in group.sprites():
        if ob.PEid != id and ob.solid: 
          obr = ob.rect
          if rect.colliderect(obr) and not(type == 1 and ob.type == 1):
            # dégat du chox
            if degat != 0:
              degatReel = (- degat * vectdist2(object.direction) / 10.0)
              if degatReel < 50: ob.looseLife2(degatReel)
              print degatReel
            # suite du mouvement
            if 0 == self.tryForcedMouvement(ob,int(dx),int(dy),level=level+1):
              return 0
    object.forcedMouvement(int(dx),int(dy))
    return 1  
